// FontExtent.cpp by katahiromz
// This file is public domain software (PDS).
#include <windows.h>
#include <windowsx.h>
#include <cstdio>

HINSTANCE g_hInstance;

extern "C"
int CALLBACK EnumFontFamExProc(
    const ENUMLOGFONTEX *   lpelf,
    const TEXTMETRIC *      lptm,
    DWORD                   FontType,
    LPARAM                  lParam)
{
    HWND hwnd = HWND(lParam);
    LPCTSTR FaceName = lpelf->elfLogFont.lfFaceName;
    if (FaceName[0] == TEXT('@'))
        return TRUE;

    INT Index = SendDlgItemMessage(hwnd, cmb1, CB_FINDSTRINGEXACT, 0, LPARAM(FaceName));
    if (Index == CB_ERR)
        SendDlgItemMessage(hwnd, cmb1, CB_ADDSTRING, 0, LPARAM(FaceName));

    return TRUE;
}

VOID UpdateFontList(HWND hwnd)
{
    LOGFONT lf;
    HDC hDC = CreateCompatibleDC(NULL);
    ZeroMemory(&lf, sizeof(lf));
    lf.lfCharSet = DEFAULT_CHARSET;
    SendDlgItemMessage(hwnd, cmb1, CB_RESETCONTENT, 0, 0);
    EnumFontFamiliesEx(hDC, &lf, FONTENUMPROC(EnumFontFamExProc), LPARAM(hwnd), 0);
    DeleteDC(hDC);

    if (SendDlgItemMessage(hwnd, cmb5, CB_GETCURSEL, 0, 0) == CB_ERR)
    {
        static const TCHAR sz[] = TEXT("");
        SendDlgItemMessage(hwnd, cmb5, CB_ADDSTRING, 0, LPARAM(sz));
    }
}

BOOL OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
    UpdateFontList(hwnd);

    SendDlgItemMessage(hwnd, cmb4, CB_ADDSTRING, 0, (LPARAM)TEXT("pt"));
    SendDlgItemMessage(hwnd, cmb4, CB_ADDSTRING, 0, (LPARAM)TEXT("px"));
    SendDlgItemMessage(hwnd, cmb4, CB_SETCURSEL, 0, 0);

    return TRUE;
}

void OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
    switch (id)
    {
    case cmb1:
    case cmb4:
        switch (codeNotify)
        {
        case CBN_EDITCHANGE: case CBN_SELENDOK:
            InvalidateRect(hwnd, NULL, TRUE);
            break;
        }
        break;
    case edt1:
    case edt2:
        if (codeNotify == EN_CHANGE)
        {
            InvalidateRect(hwnd, NULL, TRUE);
        }
        break;
    case IDOK:
        EndDialog(hwnd, IDOK);
        break;
    case IDCANCEL:
        EndDialog(hwnd, IDCANCEL);
        break;
    }
}

void OnPaint(HWND hwnd)
{
    TCHAR Buffer[128], SampleText[64], FaceName[LF_FULLFACESIZE];
    GetDlgItemText(hwnd, cmb1, FaceName, LF_FULLFACESIZE);
    GetDlgItemText(hwnd, edt2, SampleText, 64);
    INT Height = GetDlgItemInt(hwnd, edt1, NULL, FALSE);
    RECT rc;
    GetClientRect(hwnd, &rc);
    INT nComboIndex = SendDlgItemMessage(hwnd, cmb4, CB_GETCURSEL, 0, 0);

    PAINTSTRUCT ps;
    HDC hDC = BeginPaint(hwnd, &ps);
    if (hDC)
    {
        SetBkMode(hDC, OPAQUE);
        SetTextColor(hDC, 0);
        SetBkColor(hDC, RGB(255, 255, 255));

        LOGFONT lf;
        TEXTMETRIC tm;
        SIZE size;
        ZeroMemory(&lf, sizeof(lf));
        lstrcpyn(lf.lfFaceName, FaceName, LF_FACESIZE);

        if (nComboIndex == 0)
            lf.lfHeight = -MulDiv(Height, GetDeviceCaps(hDC, LOGPIXELSY), 72);
        else
            lf.lfHeight = Height;

        ZeroMemory(&tm, sizeof(tm));
        HFONT hFont = CreateFontIndirect(&lf);
        if (hFont)
        {
            HGDIOBJ hFontOld = SelectObject(hDC, hFont);

            GetTextMetrics(hDC, &tm);
            DrawText(hDC, SampleText, -1, &rc,
                DT_SINGLELINE | DT_CENTER | DT_VCENTER);
            GetTextExtentPoint(hDC, SampleText, lstrlen(SampleText), &size);

            SelectObject(hDC, hFontOld);
            DeleteObject(hFont);
        }

        wsprintf(Buffer, TEXT("Width:%ld, Height:%ld, tmHeight:%ld, ")
            TEXT("tmDescent:%ld, tmInternalLeading:%ld, ")
            TEXT("tmExternalLeading:%ld, tmAveCharWidth:%ld, ")
            TEXT("tmMaxCharWidth:%ld"),
            size.cx, size.cy, tm.tmHeight, tm.tmDescent,
            tm.tmInternalLeading, tm.tmExternalLeading,
            tm.tmAveCharWidth, tm.tmMaxCharWidth);
        DrawText(hDC, Buffer, -1, &rc, DT_WORDBREAK);

        EndPaint(hwnd, &ps);
    }
}

extern "C"
INT_PTR CALLBACK
DialogProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch (uMsg)
    {
        HANDLE_MSG(hwnd, WM_INITDIALOG, OnInitDialog);
        HANDLE_MSG(hwnd, WM_COMMAND, OnCommand);
        HANDLE_MSG(hwnd, WM_PAINT, OnPaint);
    }
    return 0;
}

extern "C"
INT WINAPI
WinMain(
    HINSTANCE   hInstance,
    HINSTANCE   hPrevInstance,
    LPSTR       lpCmdLine,
    INT         nCmdShow)
{
    g_hInstance = hInstance;
    DialogBox(hInstance, MAKEINTRESOURCE(1), NULL, DialogProc);
    return 0;
}
